@@ -1,14 +1,13 @@
 /*
- * Copyright 2014 University Corporation for Atmospheric Research
+ * Copyright 2020 University Corporation for Atmospheric Research
  *
  * This file is part of the UDUNITS-2 package.  See the file COPYRIGHT
  * in the top-level source-directory of the package for copying and
  * redistribution conditions.
  */
-#ifndef	_XOPEN_SOURCE
-#   define _XOPEN_SOURCE 500
-#endif
+#include "config.h"
 
+#include "udunits2.h"
 
 #include <float.h>
 #include <glob.h>
@@ -21,8 +20,6 @@
 #include <CUnit/CUnit.h>
 #include <CUnit/Basic.h>
 
-#include "udunits2.h"
-
 static const char*      xmlPath;
 static ut_system*	unitSystem;
 static ut_unit*		kilogram;
@@ -1112,9 +1109,9 @@ test_utAreConvertible(void)
     CU_ASSERT_EQUAL(ut_are_convertible(secondsSinceTheEpoch,
 	secondsSinceTheEpoch), 1);
     CU_ASSERT_EQUAL(ut_are_convertible(secondsSinceTheEpoch, second), 0);
-    CU_ASSERT_EQUAL(ut_are_convertible(joulePerKilogram, 
+    CU_ASSERT_EQUAL(ut_are_convertible(joulePerKilogram,
 	meterSquaredPerSecondSquared), 1);
-    CU_ASSERT_EQUAL(ut_are_convertible(joulePerKilogram, 
+    CU_ASSERT_EQUAL(ut_are_convertible(joulePerKilogram,
 	meterPerSecondSquared), 0);
 
     unit = ut_raise(radian, 2);
@@ -1195,14 +1192,14 @@ test_utGetConverter(void)
     CU_ASSERT_EQUAL(cv_convert_floats(converter, floats, 1, floats+1), floats+1);
     CU_ASSERT_EQUAL(floats[1], 0);
     doubles[0] = 0; doubles[1] = 1000;
-    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles, 1, doubles+1), 
+    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles, 1, doubles+1),
 	doubles+1);
     CU_ASSERT_EQUAL(doubles[1], 0);
     floats[0] = 0; floats[1] = 1000;
     CU_ASSERT_EQUAL(cv_convert_floats(converter, floats+1, 1, floats), floats);
     CU_ASSERT_EQUAL(floats[0], 1);
     doubles[0] = 0; doubles[1] = 1000;
-    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles+1, 1, doubles), 
+    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles+1, 1, doubles),
 	doubles);
     CU_ASSERT_EQUAL(doubles[0], 1);
     cv_free(converter);
@@ -1222,14 +1219,14 @@ test_utGetConverter(void)
     CU_ASSERT_EQUAL(cv_convert_floats(converter, floats, 1, floats+1), floats+1);
     CU_ASSERT_EQUAL(floats[1], 0);
     doubles[0] = 0; doubles[1] = 1;
-    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles, 1, doubles+1), 
+    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles, 1, doubles+1),
 	doubles+1);
     CU_ASSERT_EQUAL(doubles[1], 0);
     floats[0] = 0; floats[1] = 1;
     CU_ASSERT_EQUAL(cv_convert_floats(converter, floats+1, 1, floats), floats);
     CU_ASSERT_EQUAL(floats[0], 1000);
     doubles[0] = 0; doubles[1] = 1;
-    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles+1, 1, doubles), 
+    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles+1, 1, doubles),
 	doubles);
     CU_ASSERT_EQUAL(doubles[0], 1000);
     cv_free(converter);
@@ -1249,14 +1246,14 @@ test_utGetConverter(void)
     CU_ASSERT_EQUAL(cv_convert_floats(converter, floats, 1, floats+1), floats+1);
     CU_ASSERT_TRUE(areCloseFloats(floats[1], -273.15));
     doubles[0] = 0; doubles[1] = 273.15;
-    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles, 1, doubles+1), 
+    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles, 1, doubles+1),
 	doubles+1);
     CU_ASSERT_EQUAL(doubles[1], -273.15);
     floats[0] = 0; floats[1] = 273.15;
     CU_ASSERT_EQUAL(cv_convert_floats(converter, floats+1, 1, floats), floats);
     CU_ASSERT_TRUE(areCloseFloats(floats[0], 0));
     doubles[0] = 0; doubles[1] = 273.15;
-    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles+1, 1, doubles), 
+    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles+1, 1, doubles),
 	doubles);
     CU_ASSERT_EQUAL(doubles[0], 0);
     cv_free(converter);
@@ -1276,14 +1273,14 @@ test_utGetConverter(void)
     CU_ASSERT_EQUAL(cv_convert_floats(converter, floats, 1, floats+1), floats+1);
     CU_ASSERT_TRUE(areCloseFloats(floats[1], 273.15));
     doubles[0] = 0; doubles[1] = -273.15;
-    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles, 1, doubles+1), 
+    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles, 1, doubles+1),
 	doubles+1);
     CU_ASSERT_EQUAL(doubles[1], 273.15);
     floats[0] = 0; floats[1] = -273.15;
     CU_ASSERT_EQUAL(cv_convert_floats(converter, floats+1, 1, floats), floats);
     CU_ASSERT_TRUE(areCloseFloats(floats[0], 0));
     doubles[0] = 0; doubles[1] = -273.15;
-    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles+1, 1, doubles), 
+    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles+1, 1, doubles),
 	doubles);
     CU_ASSERT_EQUAL(doubles[0], 0);
     cv_free(converter);
@@ -1303,14 +1300,14 @@ test_utGetConverter(void)
     CU_ASSERT_EQUAL(cv_convert_floats(converter, floats, 1, floats+1), floats+1);
     CU_ASSERT_TRUE(areCloseFloats(floats[1], 32));
     doubles[0] = 0; doubles[1] = 100;
-    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles, 1, doubles+1), 
+    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles, 1, doubles+1),
 	doubles+1);
     CU_ASSERT_TRUE(areCloseDoubles(doubles[1], 32));
     floats[0] = 0; floats[1] = 100;
     CU_ASSERT_EQUAL(cv_convert_floats(converter, floats+1, 1, floats), floats);
     CU_ASSERT_TRUE(areCloseFloats(floats[0], 212));
     doubles[0] = 0; doubles[1] = 100;
-    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles+1, 1, doubles), 
+    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles+1, 1, doubles),
 	doubles);
     CU_ASSERT_TRUE(areCloseDoubles(doubles[0], 212));
     cv_free(converter);
@@ -1330,14 +1327,14 @@ test_utGetConverter(void)
     CU_ASSERT_EQUAL(cv_convert_floats(converter, floats, 1, floats+1), floats+1);
     CU_ASSERT_TRUE(areCloseFloats(floats[1], 0));
     doubles[0] = 32; doubles[1] = 212;
-    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles, 1, doubles+1), 
+    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles, 1, doubles+1),
 	doubles+1);
     CU_ASSERT_TRUE(areCloseDoubles(doubles[1], 0));
     floats[0] = 32; floats[1] = 212;
     CU_ASSERT_EQUAL(cv_convert_floats(converter, floats+1, 1, floats), floats);
     CU_ASSERT_TRUE(areCloseFloats(floats[0], 100));
     doubles[0] = 32; doubles[1] = 212;
-    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles+1, 1, doubles), 
+    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles+1, 1, doubles),
 	doubles);
     CU_ASSERT_TRUE(areCloseDoubles(doubles[0], 100));
     cv_free(converter);
@@ -1362,14 +1359,14 @@ test_utGetConverter(void)
     CU_ASSERT_EQUAL(cv_convert_floats(converter, floats, 1, floats+1), floats+1);
     CU_ASSERT_TRUE(areCloseFloats(floats[1], 10));
     doubles[0] = 10; doubles[1] = 1000;
-    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles, 1, doubles+1), 
+    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles, 1, doubles+1),
 	doubles+1);
     CU_ASSERT_TRUE(areCloseDoubles(doubles[1], 10));
     floats[0] = 10; floats[1] = 1000;
     CU_ASSERT_EQUAL(cv_convert_floats(converter, floats+1, 1, floats), floats);
     CU_ASSERT_TRUE(areCloseFloats(floats[0], 30));
     doubles[0] = 10; doubles[1] = 1000;
-    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles+1, 1, doubles), 
+    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles+1, 1, doubles),
 	doubles);
     CU_ASSERT_TRUE(areCloseDoubles(doubles[0], 30));
     cv_free(converter);
@@ -1392,14 +1389,14 @@ test_utGetConverter(void)
     CU_ASSERT_EQUAL(cv_convert_floats(converter, floats, 1, floats+1), floats+1);
     CU_ASSERT_TRUE(areCloseFloats(floats[1], 10));
     doubles[0] = 10; doubles[1] = 30;
-    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles, 1, doubles+1), 
+    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles, 1, doubles+1),
 	doubles+1);
     CU_ASSERT_TRUE(areCloseDoubles(doubles[1], 10));
     floats[0] = 10; floats[1] = 30;
     CU_ASSERT_EQUAL(cv_convert_floats(converter, floats+1, 1, floats), floats);
     CU_ASSERT_TRUE(areCloseFloats(floats[0], 1000));
     doubles[0] = 10; doubles[1] = 30;
-    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles+1, 1, doubles), 
+    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles+1, 1, doubles),
 	doubles);
     CU_ASSERT_TRUE(areCloseDoubles(doubles[0], 1000));
     cv_free(converter);
@@ -1492,14 +1489,14 @@ test_utOffsetByTime(void)
     CU_ASSERT_EQUAL(cv_convert_floats(converter, floats, 1, floats+1), floats+1);
     CU_ASSERT_TRUE(areCloseFloats(floats[1], 0));
     doubles[0] = 0; doubles[1] = 1000;
-    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles, 1, doubles+1), 
+    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles, 1, doubles+1),
 	doubles+1);
     CU_ASSERT_TRUE(areCloseDoubles(doubles[1], 0));
     floats[0] = 0; floats[1] = 1000;
     CU_ASSERT_EQUAL(cv_convert_floats(converter, floats+1, 1, floats), floats);
     CU_ASSERT_TRUE(areCloseFloats(floats[0], 1000));
     doubles[0] = 0; doubles[1] = 1000;
-    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles+1, 1, doubles), 
+    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles+1, 1, doubles),
 	doubles);
     CU_ASSERT_TRUE(areCloseDoubles(doubles[0], 1000));
     cv_free(converter);
@@ -1530,14 +1527,14 @@ test_utOffsetByTime(void)
     CU_ASSERT_EQUAL(cv_convert_floats(converter, floats, 1, floats+1), floats+1);
     CU_ASSERT_TRUE(areCloseFloats(floats[1], 0));
     doubles[0] = 0; doubles[1] = 86400;
-    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles, 1, doubles+1), 
+    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles, 1, doubles+1),
 	doubles+1);
     CU_ASSERT_TRUE(areCloseDoubles(doubles[1], 0));
     floats[0] = 0; floats[1] = 86400;
     CU_ASSERT_EQUAL(cv_convert_floats(converter, floats+1, 1, floats), floats);
     CU_ASSERT_TRUE(areCloseFloats(floats[0], 1));
     doubles[0] = 0; doubles[1] = 86400;
-    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles+1, 1, doubles), 
+    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles+1, 1, doubles),
 	doubles);
     CU_ASSERT_TRUE(areCloseDoubles(doubles[0], 1));
     cv_free(converter);
@@ -1557,14 +1554,14 @@ test_utOffsetByTime(void)
     CU_ASSERT_EQUAL(cv_convert_floats(converter, floats, 1, floats+1), floats+1);
     CU_ASSERT_TRUE(areCloseFloats(floats[1], 0));
     doubles[0] = 0; doubles[1] = 1;
-    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles, 1, doubles+1), 
+    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles, 1, doubles+1),
 	doubles+1);
     CU_ASSERT_TRUE(areCloseDoubles(doubles[1], 0));
     floats[0] = 0; floats[1] = 1;
     CU_ASSERT_EQUAL(cv_convert_floats(converter, floats+1, 1, floats), floats);
     CU_ASSERT_TRUE(areCloseFloats(floats[0], 86400));
     doubles[0] = 0; doubles[1] = 1;
-    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles+1, 1, doubles), 
+    CU_ASSERT_EQUAL(cv_convert_doubles(converter, doubles+1, 1, doubles),
 	doubles);
     CU_ASSERT_TRUE(areCloseDoubles(doubles[0], 86400));
     cv_free(converter);
@@ -2026,9 +2023,9 @@ test_xml(void)
     int                 status;
     ut_unit*            unit1;
     ut_unit*            unit2;
-    cv_converter*	    converter;
-    char*		        spec;
-    ut_unit*		    unit;
+    cv_converter*	converter;
+    char*		spec;
+    ut_unit*		unit;
 
     ut_set_error_message_handler(ut_write_to_stderr);
     xmlSystem = ut_read_xml(xmlPath);
@@ -2280,6 +2277,30 @@ test_mm2_day2_divide(void)
     ut_free_system(xmlSystem);
 }
 
+
+static void
+test_timeResolution(void)
+{
+    ut_system*  xmlSystem;
+
+    ut_set_error_message_handler(ut_write_to_stderr);
+    xmlSystem = ut_read_xml(xmlPath);
+    CU_ASSERT_PTR_NOT_NULL_FATAL(xmlSystem);
+
+    // Maximum temporal resolution:
+    const char string[] = "day since 2001-01-01 00:00:00.000000000 UTC";
+    ut_unit* unit = ut_parse(xmlSystem, string, UT_ASCII);
+    CU_ASSERT_PTR_NOT_NULL_FATAL(unit);
+
+    char buf[128];
+    int  len = ut_format(unit, buf, sizeof(buf), UT_ASCII | UT_NAMES);
+    CU_ASSERT_NOT_EQUAL_FATAL(len, -1);
+    printf("test_timeResolution() unit=\"%s\"", buf);
+    CU_ASSERT_STRING_EQUAL(buf, string);
+
+    ut_free(unit);
+}
+
 int
 main(
     const int           argc,
@@ -2331,6 +2352,7 @@ main(
 	    CU_ADD_TEST(testSuite, test_parsing);
 	    CU_ADD_TEST(testSuite, test_visitor);
 	    CU_ADD_TEST(testSuite, test_xml);
+	    CU_ADD_TEST(testSuite, test_timeResolution);
 	    /*
 	    */
 
