@@ -1,5 +1,5 @@
 /*
- * Copyright 2013 University Corporation for Atmospheric Research
+ * Copyright 2020 University Corporation for Atmospheric Research
  *
  * This file is part of the UDUNITS-2 package.  See the file COPYRIGHT
  * in the top-level source-directory of the package for copying and
@@ -14,19 +14,19 @@
  *	ProductUnit	A unit that, when it is created, contains all the
  *			BasicUnit-s that exist at the time, each raised
  *			to an integral power (that can be zero).
- *	GalileanUnit	A unit whose value is related to another unit by a 
- *			Galilean transformation (y = ax + b).  Examples include
+ *	GalileanUnit	A unit whose value is related to another unit by a
+ *			Galilean transformation (y = ax + b). Examples include
  *			"yard" and "degrees Fahrenheit".
  *	LogUnit		A unit that is related to another unit by a logarithmic
  *			transformation (y = a*log(x)).  The "Bel" is an example.
  *	TimestampUnit	A wrong-headed unit that shouldn't exist but does for
- *			backward compatibility.  It was intended to provide 
+ *			backward compatibility.  It was intended to provide
  *			similar functionality as the GalileanUnit, but for time
  *			units (e.g., "seconds since the epoch").  Unfortunately,
  *			people try to use it for more than it is capable (e.g.,
  *			days since some time on an imaginary world with only 360
  *			days per year).
- *	ut_unit		A data-structure that encapsulates ProductUnit, 
+ *	ut_unit		A data-structure that encapsulates ProductUnit,
  *			GalileanUnit, LogUnit, and TimestampUnit.
  *
  * This module is thread-compatible but not thread-safe: multi-thread access to
@@ -35,9 +35,10 @@
 
 /*LINTLIBRARY*/
 
-#ifndef	_XOPEN_SOURCE
-#   define _XOPEN_SOURCE 500
-#endif
+#include "config.h"
+
+#include "udunits2.h"		/* this module's API */
+#include "converter.h"
 
 #include <assert.h>
 #include <ctype.h>
@@ -72,9 +73,6 @@
 #include <strings.h>
 #endif
 
-#include "udunits2.h"		/* this module's API */
-#include "converter.h"
-
 typedef enum {
     PRODUCT_EQUAL = 0,		/* The units are equal -- ignoring dimensionless
 				 * basic-units */
@@ -318,7 +316,7 @@ gregorianDateToJulianDay(year, month, day)
     long	julday;	/* returned Julian day number */
 
     /*
-     * Because there is no 0 BC or 0 AD, assume the user wants the start of 
+     * Because there is no 0 BC or 0 AD, assume the user wants the start of
      * the common era if they specify year 0.
      */
     if (year == 0)
@@ -404,15 +402,20 @@ ut_encode_clock(
     int		minutes,
     double	seconds)
 {
+    if (abs(hours) >= 24 || abs(minutes) >= 60 || fabs(seconds) > 62) {
+        ut_set_status(UT_BAD_ARG);
+        return 0;
+    }
+
     return (hours*60 + minutes)*60 + seconds;
 }
 
-static int
+inline static int
 mydiv(  const double   numer,
         const unsigned denom,
         double* const  rem)
 {
-    int n = (int)(abs(numer)/denom);
+    int n = abs((int)numer)/denom;
     if (numer < 0)
         n = -n;
     *rem = numer - (long)n * (long)denom;
@@ -456,7 +459,7 @@ ut_encode_date(
 
 /*
  * Encodes a time as a double-precision value.  The convenience function is
- * equivalent to "ut_encode_date(year,month,day) + 
+ * equivalent to "ut_encode_date(year,month,day) +
  * ut_encode_clock(hour,minute,second)"
  *
  * Arguments:
@@ -591,7 +594,7 @@ static UnitOps	basicOps;
  *
  * Arguments:
  *	system		The unit-system to be associated with the new instance.
- *	isDimensionless	Whether or not the unit is dimensionless (e.g., 
+ *	isDimensionless	Whether or not the unit is dimensionless (e.g.,
  *			"radian").
  *	index		The index of the basic-unit in "system".
  * Returns:
@@ -635,7 +638,7 @@ basicNew(
 	    basicUnit->index = index;
 	    basicUnit->isDimensionless = isDimensionless;
 	    basicUnit->product = product;
-	    error = 0;
+	    error = 0;                  // Success
 	}				/* "basicUnit" allocated */
 
 	if (error)
@@ -1104,7 +1107,7 @@ productMultiply(
 	    }
 	    else {
 		static short*	powers = NULL;
-		
+
 		powers = realloc(powers, sizeof(short)*sumCount);
 
 		if (powers == NULL) {
@@ -1573,18 +1576,17 @@ productIsDimensionless(
 static UnitOps	galileanOps;
 
 
-/*
+/**
  * Returns a new unit instance.  The returned instance is not necessarily a
  * Galilean unit.
  *
- * Arguments:
- *	scale	The scale-factor for the new unit.
- *	unit	The underlying unit.  May be freed upon return.
- *	offset	The offset for the new unit.
- * Returns:
- *	NULL	Failure.  "ut_get_status()" will be:
- *		    UT_OS	Operating-system error.  See "errno".
- *	else	The newly-allocated, galilean-unit.
+ * @param[in] scale	The scale-factor for the new unit.
+ * @param[in] unit	The underlying unit.  May be freed upon return.
+ * @param[in] offset	The offset for the new unit.
+ * @retval    NULL	Failure.  `ut_get_status()` will be:
+ *		          UT_OS	Operating-system error.  See `errno`.
+ *		          UT_BAD_ARG  `scale == 0 || unit == NULL`
+ * @return              The newly-allocated, galilean-unit.
  */
 static ut_unit*
 galileanNew(
@@ -1592,48 +1594,51 @@ galileanNew(
     const ut_unit*	unit,
     double		offset)
 {
-    ut_unit*	newUnit = NULL;	/* failure */
+    ut_unit*	newUnit;
 
-    assert(scale != 0);
-    assert(unit != NULL);
-
-    if (IS_GALILEAN(unit)) {
-	scale *= unit->galilean.scale;
-	offset += (unit->galilean.scale * unit->galilean.offset) / scale;
-	unit = unit->galilean.unit;
-    }
-
-    if (areAlmostEqual(scale, 1) && areAlmostEqual(offset, 0)) {
-	newUnit = CLONE(unit);
+    if (scale == 0 || unit == NULL) {
+        ut_set_status(UT_BAD_ARG);
+        newUnit = NULL;
     }
     else {
-	GalileanUnit*	galileanUnit = malloc(sizeof(GalileanUnit));
+        if (IS_GALILEAN(unit)) {
+            scale *= unit->galilean.scale;
+            offset += (unit->galilean.scale * unit->galilean.offset) / scale;
+            unit = unit->galilean.unit;
+        }
 
-	if (galileanUnit == NULL) {
-	    ut_set_status(UT_OS);
-	    ut_handle_error_message("galileanNew(): "
-		"Couldn't allocate %lu-byte Galilean unit",
-		sizeof(GalileanUnit));
-	}
-	else {
-	    int	error = 1;
+        if (areAlmostEqual(scale, 1) && areAlmostEqual(offset, 0)) {
+            newUnit = CLONE(unit);
+        }
+        else {
+            GalileanUnit*	galileanUnit = malloc(sizeof(GalileanUnit));
 
-	    if (commonInit(&galileanUnit->common, &galileanOps,
-		    unit->common.system, GALILEAN) == 0) {
-		galileanUnit->scale = scale;
-		galileanUnit->offset = offset;
-		galileanUnit->unit = CLONE(unit);
-		error = 0;
-	    }
+            if (galileanUnit == NULL) {
+                ut_set_status(UT_OS);
+                ut_handle_error_message("galileanNew(): "
+                    "Couldn't allocate %lu-byte Galilean unit",
+                    sizeof(GalileanUnit));
+            }
+            else {
+                int	error = 1;
 
-	    if (error) {
-		free(galileanUnit);
-		galileanUnit = NULL;
-	    }
-	}				/* "galileanUnit" allocated */
+                if (commonInit(&galileanUnit->common, &galileanOps,
+                        unit->common.system, GALILEAN) == 0) {
+                    galileanUnit->scale = scale;
+                    galileanUnit->offset = offset;
+                    galileanUnit->unit = CLONE(unit);
+                    error = 0;
+                }
 
-	newUnit = (ut_unit*)galileanUnit;
-    }					/* Galilean unit necessary */
+                if (error) {
+                    free(galileanUnit);
+                    galileanUnit = NULL;
+                }
+            }				/* "galileanUnit" allocated */
+
+            newUnit = (ut_unit*)galileanUnit;
+        }					/* Galilean unit necessary */
+    } // `scale != 0 && unit != NULL`
 
     return newUnit;
 }
@@ -3345,7 +3350,7 @@ ut_divide(
  *
  * Arguments:
  *	unit	Pointer to the unit.
- *	power	The power by which to raise "unit".  Must be greater than or 
+ *	power	The power by which to raise "unit".  Must be greater than or
  *		equal to -255 and less than or equal to 255.
  * Returns:
  *	NULL	Failure.  "ut_get_status()" will be:
@@ -3372,7 +3377,7 @@ ut_raise(
 	ut_handle_error_message("ut_raise(): Invalid power argument");
     }
     else {
-	result = 
+	result =
 	    power == 0
 		? unit->common.system->one
 		: power == 1
@@ -3389,7 +3394,7 @@ ut_raise(
  *
  * Arguments:
  *	unit	Pointer to the unit.
- *	root	The root to take of "unit".  Must be greater than or 
+ *	root	The root to take of "unit".  Must be greater than or
  *		equal to 1 and less than or equal to 255.
  * Returns:
  *	NULL	Failure.  "ut_get_status()" will be:
@@ -3419,7 +3424,7 @@ ut_root(
 	ut_handle_error_message("ut_root(): Invalid root argument");
     }
     else {
-	result = 
+	result =
 	    root == 1
 		? CLONE(unit)
                 : ROOT(unit, root);
@@ -3495,7 +3500,7 @@ ut_log(
 
 /*
  * Indicates if numeric values in one unit are convertible to numeric values in
- * another unit via "ut_get_converter()".  In making this determination, 
+ * another unit via "ut_get_converter()".  In making this determination,
  * dimensionless units are ignored.
  *
  * Arguments:
@@ -3668,7 +3673,7 @@ ut_get_converter(
 		    }
 		    else {
 			cv_converter*	fromSeconds = ut_get_converter(
-			    to->common.system->second, to->timestamp.unit); 
+			    to->common.system->second, to->timestamp.unit);
 
 			if (fromSeconds == NULL) {
 			    ut_set_status(UT_OS);
