@@ -1,5 +1,5 @@
 /*
- * Copyright 2013 University Corporation for Atmospheric Research
+ * Copyright 2020 University Corporation for Atmospheric Research
  *
  * This file is part of the UDUNITS-2 package.  See the file COPYRIGHT
  * in the top-level source-directory of the package for copying and
@@ -14,8 +14,12 @@
 
 #include <config.h>
 
-#ifndef	_XOPEN_SOURCE
-#   define _XOPEN_SOURCE 500
+#include "udunits2.h"
+
+#if defined(__linux__)
+#   ifndef _GNU_SOURCE
+#       define _GNU_SOURCE
+#   endif
 #endif
 
 #include <assert.h>
@@ -34,15 +38,22 @@
 #endif
 #include <sys/stat.h>
 #include <sys/types.h>
+#if defined(__linux__)
+#include <dlfcn.h>
+#elif defined(__APPLE__)
+#define _DARWIN_C_SOURCE
+#include <dlfcn.h>
+#elif defined _WIN32
+#include <windows.h>
+#endif
+#include <expat.h>
 
 #ifndef DLL_UDUNITS2
-#define XML_STATIC
+#   define XML_STATIC
 #endif
-#include "expat.h"
-#include "udunits2.h"
 
-#ifndef _XOPEN_PATH_MAX
-#   define _XOPEN_PATH_MAX 1024 // Includes terminating NUL
+#ifndef PATH_MAX
+#   define PATH_MAX 4096 // Includes terminating NUL
 #endif
 
 #define NAME_SIZE 128
@@ -567,7 +578,7 @@ mapUnitToId(
  *
  * Arguments:
  *      unit            Pointer to the unit.
- *      id              Pointer to the identifier upon wich to base all
+ *      id              Pointer to the identifier upon which to base all
  *                      derived identifiers.
  *      encoding        The encoding of "id".
  *      isName          Whether or not "id" is a name.
@@ -610,7 +621,7 @@ mapUnitToIds(
  *
  * Arguments:
  *      unit            Pointer to the unit.
- *      name            Pointer to the name upon wich to base all derived names.
+ *      name            Pointer to the name upon which to base all derived names.
  *      encoding        The encoding of "name".
  * Returns:
  *      0               Failure.
@@ -631,7 +642,7 @@ mapUnitToNames(
  *
  * Arguments:
  *      unit            Pointer to the unit.
- *      symbol          Pointer to the symbol upon wich to base all derived
+ *      symbol          Pointer to the symbol upon which to base all derived
  *                      symbols.
  *      encoding        The encoding of "symbol".
  * Returns:
@@ -1793,7 +1804,7 @@ static void
 endImport(
     void*		data)
 {
-    char        buf[_XOPEN_PATH_MAX];
+    char        buf[PATH_MAX];
     const char* path;
 
     if (text[0] == '/') {
@@ -2076,7 +2087,7 @@ readXml(
         ut_handle_error_message("Couldn't create XML parser");
     }
     else {
-        char base[_XOPEN_PATH_MAX];
+        char base[PATH_MAX];
 #ifdef _MSC_VER
         {
             char drive[_MAX_DRIVE+1]; // Will have trailing colon
@@ -2116,6 +2127,70 @@ readXml(
 }
 
 
+/* A bit hacky but much better than modifying binaries. */
+static const char*
+default_udunits2_xml_path()
+{
+    // Returned absolute pathname of XML database
+    static char absXmlPathname[PATH_MAX];
+
+    if (absXmlPathname[0] == 0) {
+        const char* prefix = NULL; // Installation directory
+
+#       if defined(__APPLE__) || defined(__linux__)
+            Dl_info     info;
+            const char  sep = '/'; // Pathname component separator
+            char        buf[PATH_MAX];
+            const char  relXmlPathname[] = "share/udunits/udunits2.xml";
+
+            // The following should get pathname of shared-library
+            if (!dladdr(default_udunits2_xml_path, &info)) {
+                prefix = NULL;
+            }
+            else {
+                strncpy(buf, info.dli_fname, sizeof(buf))[sizeof(buf)-1] = 0;
+                memmove(buf, dirname(buf), sizeof(buf)); // "lib"
+                memmove(buf, dirname(buf), sizeof(buf)); // "lib/.."
+                prefix = buf;
+            }
+#       elif defined(_WIN32)
+            const char sep = '\\'; // Pathname component separator
+            char       buf[MAX_PATH * 4];
+            const char relXmlPathname[] = "share\\udunits\\udunits2.xml";
+            HMODULE    hModule = NULL;
+
+            GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS,
+                    (LPCTSTR)default_udunits2_xml_path, &hModule);
+
+            GetModuleFileName(hModule, buf, sizeof(buf)); // Library pathname
+            *strrchr(buf, sep) = 0; // "lib"
+            *strrchr(buf, sep) = 0; // "lib/.."
+
+            prefix = buf;
+#       endif
+
+        if (prefix == NULL) {
+            strncpy(absXmlPathname, DEFAULT_UDUNITS2_XML_PATH,
+                    sizeof(absXmlPathname));
+        } // Use default pathname
+        else {
+            int prefixLen = strlen(prefix);
+            if (prefix[prefixLen-1] == sep) {
+                --prefixLen;
+                if (prefix[prefixLen-1] == sep)
+                    --prefixLen;
+            }
+
+            snprintf(absXmlPathname, sizeof(absXmlPathname), "%.*s%c%s",
+                    prefixLen, prefix, sep, relXmlPathname);
+        } // Use computed pathname
+
+        absXmlPathname[sizeof(absXmlPathname)-1] = 0; // Ensure NUL terminated
+    } // `absXmlPathname` not set
+
+    return absXmlPathname;
+}
+
 /**
  * Returns the pathname of the XML database.
  *
@@ -2125,11 +2200,11 @@ readXml(
  *                  pathname specified by the environment variable
  *                  UDUNITS2_XML_PATH is returned if set; otherwise, the
  *                  compile-time pathname of the installed, default, unit
- *                  database is returned.
+ *                  database is returned. Caller must not free.
  */
 const char*
 ut_get_path_xml(
-	const char*	path,
+	const char* path,
 	ut_status*  status)
 {
     if (path != NULL) {
@@ -2142,7 +2217,7 @@ ut_get_path_xml(
         	*status = UT_OPEN_ENV;
     	}
     	else {
-          	path = DEFAULT_UDUNITS2_XML_PATH;
+          	path = default_udunits2_xml_path();
         	*status = UT_OPEN_DEFAULT;
     	}
     }
